/*
 *  VideoDelay.cpp
 *  kinectBasic
 *
 *  Created by Spencer Salazar on 5/14/11.
 *  Copyright 2011 Spencer Salazar. All rights reserved.
 *
 */

#include "VideoDelay.h"


VideoDelay::VideoDelay(int length) :
mLength(length),
mFrames(new CircularBuffer<ImageSourceRef>(length)),
mThreshold(0.5f)
{
    mScale = Vec3f(0.75f+Rand::randFloat(-0.1,0.1), 
                   0.75f+Rand::randFloat(-0.1,0.1), 
                   1.0f);
    
    mOffset = Vec2f(Rand::randFloat(-25,25),
                    Rand::randFloat(-25,25));
}

VideoDelay::VideoDelay(int length, Vec3f scale, Vec2f offset) :
mLength(length),
mFrames(new CircularBuffer<ImageSourceRef>(length)),
mThreshold(0.5f),
mScale(scale),
mOffset(offset)
{ }

void VideoDelay::update(ImageSourceRef nextFrame)
{
    if(mFrames->atMaximum())
    {
        ImageSourceRef currentFrame;
        mFrames->get(currentFrame);
        mTex = currentFrame;
    }
    
    mFrames->put(nextFrame);
    
    alphaFactor = 1;
}

void VideoDelay::draw(gl::GlslProg &shader)
{
    if(mTex)
    {
        gl::pushModelView();
        
        gl::translate(app::getWindowSize()/2);
        gl::scale(mScale);
        gl::translate(-app::getWindowSize()/2);
        gl::translate(mOffset);
        
        mTex.bind(2);
        
        shader.bind();
        shader.uniform( "depthTex", (int)2 );
        shader.uniform( "realtimeTex", (int)1 );
        shader.uniform( "threshold", mThreshold );
        shader.uniform( "alphaFactor", 1 );
        
        gl::draw( mTex );
        
        shader.unbind();
        
        gl::popModelView();                    
    }
}

void VideoDelay::setThreshold(float threshold)
{
    mThreshold = threshold;
}
