/*
 *  VideoRecording.cpp
 *  kinectBasic
 *
 *  Created by Spencer Salazar on 5/14/11.
 *  Copyright 2011 Spencer Salazar. All rights reserved.
 *
 */

#include "VideoRecording.h"

Recording::Recording()
{
    mTex = gl::Texture();
    mIsRecording = false;
    mIsPlayingBack = false;
    mCurrentFrame = 0;
    mLoop = false;
    mCurrentIteration = 0;
    
    mScale = Vec3f(0.75f+Rand::randFloat(-0.1,0.1), 
                   0.75f+Rand::randFloat(-0.1,0.1), 
                   1.0f);
    
    mOffset = Vec2f(Rand::randFloat(-25,25),
                    Rand::randFloat(-25,25));    
    
    setColor(NONE);
}

void Recording::setColor(Color c)
{
    mColor = c;
    
    switch(mColor)
    {
        case RED:
            mColorVector = Vec4f(1,0,0,0);
            break;
        case GREEN:
            mColorVector = Vec4f(0,1,0,0);
            break;
        case BLUE:
            mColorVector = Vec4f(0,0,1,0);
            break;
        case NONE:
            mColorVector = Vec4f(0,0,0,0);
            break;
    }
}

void Recording::startRecording()
{
    mIsRecording = true;
    mIsPlayingBack = false;
}


void Recording::addFrame(ImageSourceRef frame)
{
    if(mIsRecording)
    {
        mFrames.push_back(frame);
    }
}

void Recording::startPlayback(bool loop)
{
    mLoop = loop;
    mIsRecording = false;
    mIsPlayingBack = true;
    mCurrentFrame = 0;
    mCurrentIteration = 0;
    mUpdatesAtCurrentFrame = 0;
}

void Recording::update()
{
    if(mIsPlayingBack && mCurrentFrame < mFrames.size())
    {
        //if(mCurrentFrame % 4 == 0)
        mTex = mFrames[mCurrentFrame];
        
        alphaFactor = 1;
        
        if(mUpdatesAtCurrentFrame >= mCurrentIteration)
        {
            mCurrentFrame++;
            mUpdatesAtCurrentFrame = 0;
        }
        else
        {
            mUpdatesAtCurrentFrame++;
        }
        
        if(finishedPlayback())
        {
            if(mLoop)
            {
                mCurrentFrame = 0;
                mCurrentIteration++;
            }
            else
                mIsPlayingBack = false;
        }
    }
}

void Recording::draw(gl::GlslProg &shader)
{
    if(mIsPlayingBack && mTex)
    {
        Matrix44f realtimeMatrix;
        
        gl::pushModelView();
        
//        gl::translate(app::getWindowSize()/2);
//        gl::scale(mScale);
//        gl::translate(-app::getWindowSize()/2);
//        gl::translate(mOffset);
        
        // inverse of previous operations
//        realtimeMatrix.translate(Vec3f(-mOffset, 0));
//        realtimeMatrix.translate(Vec3f(app::getWindowSize()/2, 0));
//        realtimeMatrix.scale(Vec3f(1.0,1.0,1.0)/mScale);
//        realtimeMatrix.translate(Vec3f(-app::getWindowSize()/2, 0));
        
        realtimeMatrix.translate(Vec3f(-app::getWindowSize()/2, 0));
        realtimeMatrix.scale(Vec3f(1.0,1.0,1.0)/mScale);
        realtimeMatrix.translate(Vec3f(app::getWindowSize()/2, 0));
        realtimeMatrix.translate(Vec3f(mOffset, 0));
        
        mTex.bind(2);
        shader.bind();
        shader.uniform( "depthTex", (int)2 );
        shader.uniform( "realtimeTex", (int)1 );
        shader.uniform( "threshold", 0.75f );
        shader.uniform( "iteration", MIN(mCurrentIteration, 4) );
        shader.uniform( "alphaFactor", 1.0f );
        shader.uniform( "realtimeMatrix", realtimeMatrix );
        shader.uniform( "colorVector", mColorVector );
        gl::draw( mTex );
        shader.unbind();
        
        gl::popModelView();
    }
}


