/*
 *  VideoRecording.h
 *  kinectBasic
 *
 *  Created by Spencer Salazar on 5/14/11.
 *  Copyright 2011 Spencer Salazar. All rights reserved.
 *
 */

#include <vector>

#pragma once

class Recording
{
public:
    
    Recording();
    
    void clear() { mFrames.clear(); }
    
    void startRecording();
    
    void stopRecording() { mIsRecording = false; }
    
    bool isRecording() { return mIsRecording; }
    
    void addFrame(ImageSourceRef frame);
    void startPlayback(bool loop = false);
    
    void stopPlayback() { mIsPlayingBack = false; }
    
    bool finishedPlayback() { return mCurrentFrame >= mFrames.size(); }
    
    void update();
    void draw(gl::GlslProg &shader);
    
    int numFrames() { return mFrames.size(); }
    void setLoop(bool loop) { mLoop = loop; }
    
    enum Color
    {
        NONE = 0,
        RED,
        GREEN,
        BLUE
    };
    
    void setColor(Color c);
    
private:
    
    std::vector<ImageSourceRef> mFrames;
    
    bool mIsRecording;
    bool mIsPlayingBack;
    bool mLoop;
    int mCurrentFrame;
    int mUpdatesAtCurrentFrame;
    
    float alphaFactor;
    
    gl::Texture mTex;
    
    int mCurrentIteration;
    
    Vec2f mOffset;
    Vec3f mScale;
    
    Color mColor;
    Vec4f mColorVector;
};

